#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Following a psd tutorial found here: http://psd.fanextra.com/tutorials/text-effects/christmas-wrapped-text-effect/

# author: Pocholo
# date: 11/27/21

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugin folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import os, sys,time 
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)
					
def pm_christmas_wrapped_text():

    #Initiates the temporary state
    pdb.gimp_context_set_defaults()	
	
    #Create the background    
    img = pdb.gimp_image_new(1200, 600, RGB)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_drawable_fill(backLayer, FILL_WHITE)
    pdb.gimp_context_set_foreground((210, 234, 255))
    pdb.gimp_context_set_background((157, 207, 251))
    pdb.gimp_edit_blend(backLayer, 0, LAYER_MODE_NORMAL, 2, 100, 0, 0, FALSE, TRUE, 3, 0.20, TRUE, 600, 300, 1200, 600)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_brushes_set_brush("Snow Flakes 2")
    pdb.gimp_context_set_brush_size(120)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_spacing(2)
    pdb.gimp_context_set_brush_force(0.20)
    pdb.gimp_context_set_dynamics("Random Size")
    pdb.python_fu_density_brush_fill(img, backLayer, 35)
    
    #Image process
    pdb.gimp_display_new(img)

    pdb.gimp_image_select_rectangle(img, 2, 0, 425, 1200, 175)
    snowLayer = pdb.gimp_image_get_layer_by_name(img, "Background filled @35% with Snow Flakes 2")
    pdb.gimp_item_set_name(snowLayer, "Snow Flakes")
    pdb.gimp_edit_clear(snowLayer)
    pdb.gimp_selection_none(img)
    
    #Create the text		
    size = 350
    textW = size/0.37
    textH = size/1.05
    pdb.gimp_context_set_foreground((0, 0, 0))
    textLayer = pdb.gimp_text_layer_new(img, "XMAS", "OldSansBlack Medium", size, PIXELS)
    pdb.gimp_image_add_layer(img, textLayer, 0)
    pdb.gimp_layer_add_alpha(textLayer)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 20)
    
    #Resize text
    pdb.gimp_layer_scale (textLayer, textW, textH, TRUE)
    
    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    pdb.gimp_layer_set_mode(textLayer, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(textLayer, 100)  
    #Rename the text layer
    pdb.gimp_item_set_name(textLayer, "Text")
    
    #4 letters 4 color
    pdb.gimp_context_set_foreground((230, 0, 0))
    pdb.gimp_edit_bucket_fill_full(textLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 15, FALSE, FALSE, 0, 230, 300)
    pdb.gimp_context_set_foreground((65, 130, 0))
    pdb.gimp_edit_bucket_fill_full(textLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 15, FALSE, FALSE, 0, 480, 380)
    pdb.gimp_context_set_foreground((255, 153, 0))
    pdb.gimp_edit_bucket_fill_full(textLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 15, FALSE, FALSE, 0, 720, 235)
    pdb.gimp_context_set_foreground((60, 27, 106))
    pdb.gimp_edit_bucket_fill_full(textLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 15, FALSE, FALSE, 0, 960, 316)

    #Gradient overlay
    gradLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Gradient overlay", 100, LAYER_MODE_MULTIPLY)
    pdb.gimp_image_add_layer(img, gradLayer, 0)
    pdb.gimp_drawable_fill(gradLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_context_set_background("White")
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_edit_blend(gradLayer, 
                                0,  #blend - fg to bg
                                0,  #layer mode - normal
                                1,  #gradient type - bi-linear
                              100,  #opacity
                                0,  #offset 
                                0,  #repeat - none
                             TRUE,  #reverse
                             TRUE,  #supersample
                                3,  #max depth
                             0.20,  #threshold
                             TRUE,  #dither
                              600,  #1x
                              325,  #1x
                              600,  #2y
                              520)  #2y
    
    #Inner glow
    inglowLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Inner Glow", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, inglowLayer, 0)
    pdb.gimp_drawable_fill(inglowLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_edit_bucket_fill(inglowLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_shrink(img, 7)
    pdb.gimp_edit_clear(inglowLayer)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.plug_in_gauss(img, inglowLayer, 30, 30, 1)
    pdb.gimp_selection_none(img)
    
    #Texture
    textureLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Texture", 60, LAYER_MODE_OVERLAY)
    pdb.gimp_image_add_layer(img, textureLayer, 2)
    pdb.gimp_drawable_fill(textureLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_context_set_pattern("Crinkled Paper")
    pdb.gimp_edit_bucket_fill_full(textureLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 15, FALSE, FALSE, 0, 0, 0)
    #pdb.gimp_selection_none(img)
    
    #Ribbons
    ribbLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Ribbons", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, ribbLayer, 0)
    pdb.gimp_drawable_fill(ribbLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("2. Block 01")
    pdb.gimp_context_set_brush_size(250)
    pdb.gimp_context_set_brush_aspect_ratio(12)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    
    #Red ribbon
    pdb.gimp_context_set_foreground((160, 0, 0))
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [225, 305])
    pdb.gimp_context_set_brush_angle(90)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [225, 305])
    
    #Green ribbon
    pdb.gimp_context_set_foreground((88, 176, 0))
    pdb.gimp_context_set_brush_size(110)
    pdb.gimp_context_set_brush_aspect_ratio(5)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [480, 380])
    pdb.gimp_context_set_brush_size(250)
    pdb.gimp_context_set_brush_aspect_ratio(12)
    pdb.gimp_context_set_brush_angle(90)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [480, 380])
    
    #Gold ribbon
    pdb.gimp_context_set_foreground((187, 126, 35))
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [745, 350])
    pdb.gimp_context_set_brush_size(110)
    pdb.gimp_context_set_brush_aspect_ratio(5)
    pdb.gimp_context_set_brush_angle(90)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [745, 350])
    
    #Blue ribbon
    pdb.gimp_context_set_foreground((55, 2, 130))
    pdb.gimp_context_set_brush_size(250)
    pdb.gimp_context_set_brush_aspect_ratio(12)
    pdb.gimp_context_set_brush_angle(20)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [970, 305])
    pdb.gimp_context_set_brush_size(110)
    pdb.gimp_context_set_brush_aspect_ratio(5)
    pdb.gimp_context_set_brush_angle(112)
    pdb.gimp_paintbrush_default(ribbLayer, 2, [970, 305])
    pdb.gimp_selection_none(img)
    
    #Ribbon edge color
    edgeLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Ribbon edge color", 80, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, edgeLayer, 0)
    pdb.gimp_drawable_fill(edgeLayer, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground((224, 186, 0))
    pdb.gimp_image_select_item(img, 2, ribbLayer)
    pdb.gimp_edit_bucket_fill(edgeLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_shrink(img, 2)
    pdb.gimp_edit_clear(edgeLayer)
    pdb.gimp_selection_none(img)
    
    #Bow 1
    redLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Bow Red", 80, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, redLayer, 0)
    pdb.gimp_drawable_fill(redLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Christmad red-gold bow")
    pdb.gimp_context_set_brush_size(81)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_force(1)
    #pdb.gimp_context_set_paint_method("gimp-paintbrush")
    pdb.gimp_context_set_dynamics("Dynamics Off")  
    pdb.gimp_paintbrush_default(redLayer, 2, [225, 305])
    pdb.gimp_hue_saturation(redLayer, 0, 0, 22, 100)
    
    #Bow 2
    greenLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Bow Green", 80, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, greenLayer, 0)
    pdb.gimp_drawable_fill(greenLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Christmad red-gold bow")
    pdb.gimp_paintbrush_default(greenLayer, 2, [480, 380])
    pdb.gimp_hue_saturation(greenLayer, 0, 120, -60, 100)
    
    #Bow 3
    goldLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Bow Gold", 80, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, goldLayer, 0)
    pdb.gimp_drawable_fill(goldLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Christmad red-gold bow")
    pdb.gimp_paintbrush_default(goldLayer, 2, [745, 350])
    pdb.gimp_hue_saturation(goldLayer, 0, 44, 15, -20)
    
    #Bow 4
    blueLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Bow Blue", 80, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, blueLayer, 0)
    pdb.gimp_drawable_fill(blueLayer, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("Christmad red-gold bow")
    pdb.gimp_paintbrush_default(blueLayer, 2, [970, 305])
    pdb.gimp_hue_saturation(blueLayer, 0, -112, -45, -20)
    
    #Bow
    blueLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Bow Blue", 80, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, blueLayer, 0)
    pdb.gimp_drawable_fill(blueLayer, FILL_TRANSPARENT)
    pdb.gimp_item_set_visible(backLayer, FALSE)
    
    #Text reflection
    pdb.gimp_item_set_visible(backLayer, FALSE)
    reflectionLayer = pdb.gimp_layer_new_from_visible(img, img, "Reflection")
    pdb.gimp_image_add_layer(img, reflectionLayer, 0)
    pdb.gimp_layer_set_mode(reflectionLayer, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_item_set_visible(backLayer, TRUE)
    reflectionLayer = pdb.gimp_item_transform_flip_simple(reflectionLayer, ORIENTATION_VERTICAL, TRUE, 0)
    reflectionLayer = pdb.gimp_item_transform_translate(reflectionLayer, 0, 225)
    pdb.gimp_layer_resize_to_image_size(reflectionLayer)   
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_context_set_background("White") 
    newMask = pdb.gimp_layer_create_mask(reflectionLayer, ADD_MASK_WHITE)
    pdb.gimp_layer_add_mask(reflectionLayer, newMask)
    pdb.gimp_edit_blend(newMask, 
                              0,  #blend mode - fg to bg rgb
              LAYER_MODE_NORMAL,  #paint mode 
                              0,  #gradient type - linear
                            100,  #opacity 
                              0,  #offset
                              0,  #repeat - none
                           TRUE,  #reverse
                           TRUE,  #supe-samle
                              3,  #max depth
                           0.20,  #threshold                            
                           TRUE,  #dither
                            600,  #x1
                            310,  #y1
                            600,  #x2
                            600)  #y2
    pdb.gimp_layer_remove_mask(reflectionLayer, MASK_APPLY)
    pdb.gimp_image_set_active_layer(img, backLayer)
    
    #Make active the bottom layer and done
    pdb.gimp_image_set_active_layer(img, backLayer)
    #Set the defaults
    pdb.gimp_context_set_defaults()
       
register(
    "pm_christmas_wrapped_text",
    "Creates Christmas present text effect",
    "Creates Christmas present text effect",
    "Pocholo",
    "Pocholo",
    "2021",
    "Christmas Wrap Text Effect",
    "",
    [ 
    ],
    [],
    pm_christmas_wrapped_text, menu="<Image>/Pocholo-scripts/Christmas Present Text Effect",
    domain=("gimp20", gimp.locale_directory))
main()							